@@ -28,6 +28,9 @@
 # endif
 #endif
 
+#ifndef __GNUC__
+#define  __attribute__(x)  /*NOTHING*/
+#endif
 
 #ifdef NDEBUG
 
@@ -83,27 +86,6 @@ typedef enum {
 } WMNotificationCoalescing;
 
 
-/* The possible states for connections */
-typedef enum {
-    WCNotConnected=0,
-    WCListening,
-    WCInProgress,
-    WCFailed,
-    WCConnected,
-    WCTimedOut,
-    WCDied,
-    WCClosed
-} WMConnectionState;
-
-
-/* The possible states for connection timeouts */
-typedef enum {
-    WCTNone=0,
-    WCTWhileOpening,
-    WCTWhileSending
-} WMConnectionTimeoutState;
-
-
 
 enum {
     WBNotFound = INT_MIN, /* element was not found in WMBag   */
@@ -135,11 +117,12 @@ typedef void WMInputProc(int fd, int mask, void *clientData);
 
 
 typedef int WMCompareDataProc(const void *item1, const void *item2);
+typedef void WMTreeWalkProc(WMTreeNode *aNode, void *data);
 
 typedef void WMFreeDataProc(void *data);
 
 /* Used by WMBag or WMArray for matching data */
-typedef int WMMatchDataProc(void *item, void *cdata);
+typedef int WMMatchDataProc(const void *item, const void *cdata);
 
 
 
@@ -174,53 +157,41 @@ typedef int WMArrayIterator;
 typedef void *WMBagIterator;
 
 
-#if 0
-typedef struct {
-    char character;		       /* the escape character */
-    char *value;		       /* value to place */
-} WMSEscapes;
-#endif
-
-
-/* The connection callbacks */
-typedef struct ConnectionDelegate {
-    void *data;
-
-    void (*canResumeSending)(struct ConnectionDelegate *self, WMConnection *cPtr);
-
-    void (*didCatchException)(struct ConnectionDelegate *self, WMConnection *cPtr);
-
-    void (*didDie)(struct ConnectionDelegate *self, WMConnection *cPtr);
-
-    void (*didInitialize)(struct ConnectionDelegate *self, WMConnection *cPtr);
-
-    void (*didReceiveInput)(struct ConnectionDelegate *self, WMConnection *cPtr);
-
-    void (*didTimeout)(struct ConnectionDelegate *self, WMConnection *cPtr);
-
-} ConnectionDelegate;
-
-
 typedef void WMNotificationObserverAction(void *observerData,
                                           WMNotification *notification);
 
 
+/* ---[ WINGs/memory.c ]-------------------------------------------------- */
 
-/*......................................................................*/
+void* wmalloc(size_t size);
+void* wrealloc(void *ptr, size_t newsize);
+void wfree(void *ptr);
+
+void wrelease(void *ptr);
+void* wretain(void *ptr);
 
 typedef void waborthandler(int);
 
 waborthandler* wsetabort(waborthandler* handler);
 
+/* ---[ WINGs/error.c ]--------------------------------------------------- */
 
-/* don't free the returned string */
-char* wstrerror(int errnum);
+enum {
+	WMESSAGE_TYPE_MESSAGE,
+	WMESSAGE_TYPE_WARNING,
+	WMESSAGE_TYPE_ERROR,
+	WMESSAGE_TYPE_FATAL
+};
 
-void wmessage(const char *msg, ...);
-void wwarning(const char *msg, ...);
-void wfatal(const char *msg, ...);
-void wsyserror(const char *msg, ...);
-void wsyserrorwithcode(int error, const char *msg, ...);
+#define wmessage(fmt, args...) __wmessage( __func__, __FILE__, __LINE__, WMESSAGE_TYPE_MESSAGE, fmt, ## args)
+#define wwarning(fmt, args...) __wmessage( __func__, __FILE__, __LINE__, WMESSAGE_TYPE_WARNING, fmt, ## args)
+#define werror(fmt, args...) __wmessage( __func__, __FILE__, __LINE__, WMESSAGE_TYPE_ERROR, fmt, ## args)
+#define wfatal(fmt, args...) __wmessage( __func__, __FILE__, __LINE__, WMESSAGE_TYPE_FATAL, fmt, ## args)
+
+void __wmessage(const char *func, const char *file, int line, int type, const char *msg, ...)
+	__attribute__((__format__(printf,5,6)));
+
+/* ---[ WINGs/findfile.c ]------------------------------------------------ */
 
 char* wfindfile(char *paths, char *file);
 
@@ -230,19 +201,20 @@ char* wfindfileinarray(WMPropList* array, char *file);
 
 char* wexpandpath(char *path);
 
+int wcopy_file(char *toPath, char *srcFile, char *destFile);
+
 /* don't free the returned string */
-char* wgethomedir();
+char* wgethomedir(void);
 
-void* wmalloc(size_t size);
-void* wrealloc(void *ptr, size_t newsize);
-void wfree(void *ptr);
+/* ---[ WINGs/proplist.c ]------------------------------------------------ */
 
+int wmkdirhier(const char *path);
+int wrmdirhier(const char *path);
 
-void wrelease(void *ptr);
-void* wretain(void *ptr);
+/* ---[ WINGs/string.c ]-------------------------------------------------- */
 
-char* wstrdup(char *str);
-char* wstrndup(char *str, size_t len);
+char *wstrdup(const char *str);
+char* wstrndup(const char *str, size_t len);
 
 /* Concatenate str1 with str2 and return that in a newly malloc'ed string.
  * str1 and str2 can be any strings including static and constant strings.
@@ -259,6 +231,9 @@ char* wstrconcat(char *str1, char *str2);
  * so always assign the returned address to avoid dangling pointers. */
 char* wstrappend(char *dst, char *src);
 
+size_t wstrlcpy(char *, const char *, size_t);
+size_t wstrlcat(char *, const char *, size_t);
+
 
 void wtokensplit(char *command, char ***argv, int *argc);
 
@@ -268,27 +243,23 @@ char* wtokenjoin(char **list, int count);
 
 void wtokenfree(char **tokens, int count);
 
-char* wtrimspace(char *s);
+char* wtrimspace(const char *s);
 
+/* transform `s' so that the result is safe to pass to the shell as an argument.
+ * returns a newly allocated string.
+ * with very heavy inspirations from NetBSD's shquote(3).
+ */
+char *wshellquote(const char *s);
+
+/* ---[ WINGs/wmisc.c ]--------------------------------------------------- */
 
 WMRange wmkrange(int start, int count);
-#ifdef ANSI_C_DOESNT_LIKE_IT_THIS_WAY
-#define wmkrange(position, count) (WMRange){(position), (count)}
-#endif
 
+/* ---[ WINGs/usleep.c ]-------------------------------------------------- */
 
-char* wusergnusteppath();
+void wusleep(unsigned int usec);
 
-char* wdefaultspathfordomain(char *domain);
-
-void wusleep(unsigned int microsec);
-
-#if 0
-int wsprintesc(char *buffer, int length, char *format, WMSEscapes **escapes,
-               int count);
-#endif
-
-/*......................................................................*/
+/* ---[ WINGs/handlers.c ]------------------------------------------------ */
 
 /* Event handlers: timer, idle, input */
 
@@ -317,9 +288,9 @@ void WMDeleteInputHandler(WMHandlerID handlerID);
  * This function will handle all input/timer/idle events, then return.
  */
 
-void WHandleEvents();
+void WHandleEvents(void);
 
-/*......................................................................*/
+/* ---[ WINGs/hashtable.c ]----------------------------------------------- */
 
 
 WMHashTable* WMCreateHashTable(WMHashTableCallbacks callbacks);
@@ -380,7 +351,7 @@ extern const WMHashTableCallbacks WMStringPointerHashCallbacks;
 /* keys are strings, but they are not copied */
 
 
-/*......................................................................*/
+/* ---[ WINGs/array.c ]--------------------------------------------------- */
 
 /*
  * WMArray use an array to store the elements.
@@ -416,8 +387,6 @@ void WMAppendArray(WMArray *array, WMArray *other);
 /* add will place the element at the end of the array */
 void WMAddToArray(WMArray *array, void *item);
 
-#define WMPushInArray(array, item) WMAddToArray(array, item)
-
 /* insert will increment the index of elements after it by 1 */
 void WMInsertInArray(WMArray *array, int index, void *item);
 
@@ -482,7 +451,7 @@ void* WMArrayPrevious(WMArray *array, WMArrayIterator *iter);
     for (var = WMArrayLast(array, &(i)); (i) != WANotFound; \
     var = WMArrayPrevious(array, &(i)))
 
-/*..........................................................................*/
+/* ---[ WINGs/bagtree.c ]------------------------------------------------- */
 
 /*
  * Tree bags use a red-black tree for storage.
@@ -574,7 +543,7 @@ int WMBagIndexForIterator(WMBag *bag, WMBagIterator ptr);
 
 
 
-/*-------------------------------------------------------------------------*/
+/* ---[ WINGs/data.c ]---------------------------------------------------- */
 
 /* WMData handling */
 
@@ -643,8 +612,7 @@ void WMSetDataFormat(WMData *aData, unsigned format);
 unsigned WMGetDataFormat(WMData *aData);
 /* Storing data */
 
-
-/*--------------------------------------------------------------------------*/
+/* ---[ WINGs/tree.c ]---------------------------------------------------- */
 
 /* Generic Tree and TreeNode */
 
@@ -683,11 +651,17 @@ void WMSortTree(WMTreeNode *aNode, WMCompareDataProc *comparer);
 /* Returns the first node which matches node's data with cdata by 'match' */
 WMTreeNode* WMFindInTree(WMTreeNode *aTree, WMMatchDataProc *match, void *cdata);
 
+/* Returns the first node where node's data matches cdata by 'match' and node is
+ * at most `limit' depths down from `aTree'. */
+WMTreeNode *WMFindInTreeWithDepthLimit(WMTreeNode * aTree, WMMatchDataProc * match, void *cdata, int limit);
+
 /* Returns first tree node that has data == cdata */
 #define WMGetFirstInTree(aTree, cdata) WMFindInTree(aTree, NULL, cdata)
 
+/* Walk every node of aNode with `walk' */
+void WMTreeWalk(WMTreeNode *aNode, WMTreeWalkProc * walk, void *data, Bool DepthFirst);
 
-/*--------------------------------------------------------------------------*/
+/* ---[ WINGs/data.c ]---------------------------------------------------- */
 
 
 WMNotification* WMCreateNotification(const char *name, void *object, void *clientData);
@@ -733,7 +707,7 @@ void WMEnqueueCoalesceNotification(WMNotificationQueue *queue,
                                    unsigned coalesceMask);
 
 
-/*......................................................................*/
+/* ---[ WINGs/proplist.c ]------------------------------------------------ */
 
 /* Property Lists handling */
 
@@ -833,9 +807,15 @@ char* WMGetPropListDescription(WMPropList *plist, Bool indented);
 
 WMPropList* WMReadPropListFromFile(char *file);
 
-Bool WMWritePropListToFile(WMPropList *plist, char *path, Bool atomically);
+Bool WMWritePropListToFile(WMPropList *plist, char *path);
 
-/*......................................................................*/
+/* ---[ WINGs/userdefaults.c ]-------------------------------------------- */
+
+char* wusergnusteppath(void);
+
+char* wdefaultspathfordomain(char *domain);
+
+char* wglobaldefaultspathfordomain(const char *domain);
 
 WMUserDefaults* WMGetStandardUserDefaults(void);
 
@@ -888,135 +868,6 @@ extern char *WMUserDefaultsDidChangeNotification;
 
 /*-------------------------------------------------------------------------*/
 
-/* WMHost: host handling */
-
-WMHost* WMGetCurrentHost();
-
-WMHost* WMGetHostWithName(char* name);
-
-WMHost* WMGetHostWithAddress(char* address);
-
-WMHost* WMRetainHost(WMHost *hPtr);
-
-void WMReleaseHost(WMHost *hPtr);
-
-/*
- * Host cache management
- * If enabled, only one object representing each host will be created, and
- * a shared instance will be returned by all methods that return a host.
- * Enabled by default.
- */
-void WMSetHostCacheEnabled(Bool flag);
-
-Bool WMIsHostCacheEnabled();
-
-void WMFlushHostCache();
-
-/*
- * Compare hosts: Hosts are equal if they share at least one address
- */
-Bool WMIsHostEqualToHost(WMHost* hPtr, WMHost* anotherHost);
-
-/*
- * Host names.
- * WMGetHostName() will return first name (official) if a host has several.
- * WMGetHostNames() will return a R/O WMArray with all the names of the host.
- */
-char* WMGetHostName(WMHost* hPtr);
-
-/* The returned array is R/O. Do not modify it, and do not free it! */
-WMArray* WMGetHostNames(WMHost* hPtr);
-
-/*
- * Host addresses.
- * Addresses are represented as "Dotted Decimal" strings, e.g. "192.42.172.1"
- * WMGetHostAddress() will return an arbitrary address if a host has several.
- * WMGetHostAddresses() will return a R/O WMArray with all addresses of the host.
- */
-char* WMGetHostAddress(WMHost* hPtr);
-
-/* The returned array is R/O. Do not modify it, and do not free it! */
-WMArray* WMGetHostAddresses(WMHost* hPtr);
-
-
-/*-------------------------------------------------------------------------*/
-
-/* WMConnection functions */
-
-WMConnection* WMCreateConnectionAsServerAtAddress(char *host, char *service,
-                                                  char *protocol);
-
-WMConnection* WMCreateConnectionToAddress(char *host, char *service,
-                                          char *protocol);
-
-WMConnection* WMCreateConnectionToAddressAndNotify(char *host, char *service,
-                                                   char *protocol);
-
-void WMCloseConnection(WMConnection *cPtr);
-
-void WMDestroyConnection(WMConnection *cPtr);
-
-WMConnection* WMAcceptConnection(WMConnection *listener);
-
-/* Release the returned data! */
-WMData* WMGetConnectionAvailableData(WMConnection *cPtr);
-
-int WMSendConnectionData(WMConnection *cPtr, WMData *data);
-
-Bool WMEnqueueConnectionData(WMConnection *cPtr, WMData *data);
-
-#define WMFlushConnection(cPtr)  WMSendConnectionData((cPtr), NULL)
-
-void WMSetConnectionDelegate(WMConnection *cPtr, ConnectionDelegate *delegate);
-
-/* Connection info */
-
-char* WMGetConnectionAddress(WMConnection *cPtr);
-
-char* WMGetConnectionService(WMConnection *cPtr);
-
-char* WMGetConnectionProtocol(WMConnection *cPtr);
-
-Bool WMSetConnectionNonBlocking(WMConnection *cPtr, Bool flag);
-
-Bool WMSetConnectionCloseOnExec(WMConnection *cPtr, Bool flag);
-
-void WMSetConnectionShutdownOnClose(WMConnection *cPtr, Bool flag);
-
-void* WMGetConnectionClientData(WMConnection *cPtr);
-
-void WMSetConnectionClientData(WMConnection *cPtr, void *data);
-
-unsigned int WMGetConnectionFlags(WMConnection *cPtr);
-
-void WMSetConnectionFlags(WMConnection *cPtr, unsigned int flags);
-
-int WMGetConnectionSocket(WMConnection *cPtr);
-
-WMConnectionState WMGetConnectionState(WMConnection *cPtr);
-
-WMConnectionTimeoutState WMGetConnectionTimeoutState(WMConnection *cPtr);
-
-WMArray* WMGetConnectionUnsentData(WMConnection *cPtr);
-
-#define WMGetConnectionQueuedData(cPtr) WMGetConnectionUnsentData(cPtr)
-
-
-/*
- * Passing timeout==0 in the SetTimeout functions below, will reset that
- * timeout to its default value.
- */
-
-/* The default timeout inherited by all WMConnection operations, if none set */
-void WMSetConnectionDefaultTimeout(unsigned int timeout);
-
-/* Global timeout for all WMConnection objects, for opening a new connection */
-void WMSetConnectionOpenTimeout(unsigned int timeout);
-
-/* Connection specific timeout for sending out data */
-void WMSetConnectionSendTimeout(WMConnection *cPtr, unsigned int timeout);
-
-
 /* Global variables */
 
 extern int WCErrorCode;
@@ -1024,8 +875,6 @@ extern int WCErrorCode;
 
 /*-------------------------------------------------------------------------*/
 
-
-
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
