@@ -1,8 +1,8 @@
 /*
  *  Window Maker window manager
- * 
- *  Copyright (c) 1997, 1998 Alfredo K. Kojima
- * 
+ *
+ *  Copyright (c) 1997-2003 Alfredo K. Kojima
+ *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
@@ -13,17 +13,15 @@
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
- *  USA.
+ *  You should have received a copy of the GNU General Public License along
+ *  with this program; if not, write to the Free Software Foundation, Inc.,
+ *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #ifndef WMRESOURCES_H_
 #define WMRESOURCES_H_
 
-WFont *wLoadFont(char *font_name);
-void wFreeFont(WFont *font);
 int wGetColor(WScreen *scr, char *color_name, XColor *color);
 void wFreeColor(WScreen *scr, unsigned long pixel);
+
 #endif
