@@ -30,20 +30,15 @@
 #include <string.h>
 #include <time.h>
 
-
 #include "wraster.h"
 
+extern Bool RSaveXPM(RImage * image, char *filename);
 
-extern Bool RSaveXPM(RImage *image, char *filename);
-
-
-Bool
-RSaveImage(RImage *image, char *filename, char *format)
+Bool RSaveImage(RImage * image, char *filename, char *format)
 {
-    if (strcmp(format, "XPM")!=0) {
-        RErrorCode = RERR_BADFORMAT;
-        return False;
-    }
-    return RSaveXPM(image, filename);
+	if (strcmp(format, "XPM") != 0) {
+		RErrorCode = RERR_BADFORMAT;
+		return False;
+	}
+	return RSaveXPM(image, filename);
 }
-
